package cn.com.qmth.examcloud.exchange.dao.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import cn.com.qmth.examcloud.common.dto.em.ExamStudent;
import cn.com.qmth.examcloud.exchange.dao.OutletScoreDao;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletScore;

/**
 * @author  	chenken
 * @date    	2018年1月3日 下午4:02:31
 * @company 	QMTH
 * @description OutletScoreRepoImpl.java
 */
@Repository("outletScoreDao")
public class OutletScoreDaoImpl implements OutletScoreDao{

	private static final Logger log = LoggerFactory.getLogger(OutletScoreDaoImpl.class);
	
	public JdbcTemplate jdbcTemplate;
	
	@Override
	public List<OutletScore> sydxGetExamScoreById(Long scoreId) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "
                +" score.id 					score_id,"
                +" score.total_score 			total_score,"
                +" score.objective_score 		objective_score,"
                +" score.subjective_score 		subjective_score," 
                +" record.student_name 			student_name,"
                +" record.student_code 			student_code,"
                +" record.identity_number 		identity_number," 
                +" stu.course_name 				course_name,"
                +" stu.course_code 				course_code,"
                +" data.end_time 				end_time " 
                +" from ec_oe_exam_score score " 
                +" left join ec_oe_exam_record_data data on score.exam_record_data_id = data.id" 
                +" left join ec_oe_exam_record record on data.exam_record_id = record.id"
                +" left join ec_oe_exam_student stu on record.exam_student_id = stu.id"
                +" where score.id = "+scoreId);
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<OutletScore>() {
			@Override
			public OutletScore mapRow(ResultSet rs, int arg1) throws SQLException {
				return getOutletScoreByResultSet(rs);
			}
		});
	}
	

	@Override
	public List<OutletScore> getExamScoreById(Long scoreId) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "
	                +" score.id 				 score_id,"
	                +" score.total_score 		 total_score,"
	                +" score.objective_score 	 objective_score,"
	                +" score.subjective_score 	 subjective_score," 
	                +" record.student_name 		 student_name,"
	                +" record.student_code 		 student_code,"
	                +" record.identity_number 	 identity_number," 
	                +" stu.course_name 		 	 course_name,"
	                +" stu.course_code 		 	 course_code,"
	                +" data.start_time			 start_time,"
	                +" data.end_time 			 end_time," 
	                +" data.face_success_percent succ_percent,"
	                +" data.is_illegality 		 is_illegality "
	                +" from ec_oe_exam_score score"
	                +" left join ec_oe_exam_record_data data on score.exam_record_data_id = data.id" 
	                +" left join ec_oe_exam_record record on data.exam_record_id = record.id" 
	                +" left join ec_oe_exam_student stu on record.exam_student_id = stu.id"
	                +" where score.id = "+scoreId);
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<OutletScore>() {
			@Override
			public OutletScore mapRow(ResultSet rs, int arg1) throws SQLException {
				return getExamScoreByIdResult(rs);
			}
		});
	}

	private OutletScore getExamScoreByIdResult(ResultSet rs) throws SQLException{
		OutletScore outletScore = new OutletScore();
		outletScore.setScoreId(rs.getLong("score_id"));
		outletScore.setStudentName(rs.getString("student_name"));
		outletScore.setStudentCode(rs.getString("student_code"));
		outletScore.setIdentityNumber(rs.getString("identity_number"));
		outletScore.setCourseName(rs.getString("course_name"));
		outletScore.setCourseCode(rs.getString("course_code"));
		
		outletScore.setTotalScore(rs.getDouble("total_score"));
		outletScore.setObjectiveScore(rs.getDouble("objective_score"));
		outletScore.setSubjectiveScore(rs.getDouble("subjective_score"));
		
		outletScore.setStartTime(rs.getString("start_time"));
		outletScore.setEndTime(rs.getString("end_time"));
		outletScore.setSuccessRate(rs.getDouble("succ_percent"));
		Integer isIllegality = rs.getInt("is_illegality");
		outletScore.setIsBreachThePrinciple(isIllegality == 1?true:false);
		return outletScore;
	}

	private OutletScore getOutletScoreByResultSet(ResultSet rs) throws SQLException{
		OutletScore outletScore = new OutletScore();
		outletScore.setScoreId(rs.getLong("score_id"));
		outletScore.setStudentName(rs.getString("student_name"));
		outletScore.setStudentCode(rs.getString("student_code"));
		outletScore.setIdentityNumber(rs.getString("identity_number"));
		outletScore.setTotalScore(rs.getDouble("total_score"));
		outletScore.setObjectiveScore(rs.getDouble("objective_score"));
		outletScore.setSubjectiveScore(rs.getDouble("subjective_score"));
		outletScore.setCourseName(rs.getString("course_name"));
		outletScore.setCourseCode(rs.getString("course_code"));
		outletScore.setEndTime(rs.getString("end_time"));
		return outletScore;
	}

	@Override
	public List<ExamStudent> findExamStudents(Long examId) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "+
						 " stu.id 				id,"+
						 " stu.exam_id			exam_id,"+
						 " stu.student_name		student_name,"+
						 " stu.student_code 	student_code,"+
						 " stu.identity_number 	identity_number,"+
						 " stu.course_name 		course_name,"+
						 " stu.course_code 		course_code,"+
						 " stu.finished			finished"+
					" from ec_oe_exam_student stu "+
					" where stu.exam_id = "+examId);
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudent>() {
			@Override
			public ExamStudent mapRow(ResultSet rs, int arg1) throws SQLException {
				return getExamStudentByResultSet(rs);
			}
		});
	}
	
	private ExamStudent getExamStudentByResultSet(ResultSet rs) throws SQLException{
		ExamStudent examStudent = new ExamStudent();
		examStudent.setId(rs.getLong("id"));
		examStudent.setExamId(rs.getLong("exam_id"));
		examStudent.setName(rs.getString("student_name"));
		examStudent.setStudentCode(rs.getString("student_code"));
		examStudent.setIdentityNumber(rs.getString("identity_number"));
		examStudent.setCourseCode(rs.getString("course_code"));
		examStudent.setCourseName(rs.getString("course_name"));
		examStudent.setFinished(rs.getBoolean("finished"));
		return examStudent;
	}

	@Override
	public List<ExamStudent> findExamStudents(List<String> identityNumbers,Long rootOrgId) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "+
						 " stu.id 				id,"+
						 " stu.exam_id 			exam_id,"+
						 " stu.student_name 	student_name,"+
						 " stu.student_code 	student_code,"+
						 " stu.identity_number 	identity_number,"+
						 " stu.course_name 		course_name,"+
						 " stu.course_code 		course_code,"+
						 " stu.finished			finished"+
					" from ec_oe_exam_student stu "+
					" where 1=1 ");
		StringBuffer numberString = new StringBuffer();
		for(String number:identityNumbers){
			numberString.append("'"+number+"',");
		}
		String numbers = numberString.toString();
		numbers = numbers.substring(0, numbers.lastIndexOf(","));
		sql.append(" and stu.identity_number in ("+numbers+") and stu.root_org_id = "+rootOrgId );
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudent>() {
			@Override
			public ExamStudent mapRow(ResultSet rs, int arg1) throws SQLException {
				return getExamStudentByResultSet(rs);
			}
		});
	}

	@Override
	public List<ExamStudent> findExamStudents(Long examId,String courseCode, List<String> studentCodes) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "+
					     " stu.id 				id,"+
					     " stu.exam_id			exam_id,"+
						 " stu.student_name 	student_name,"+
						 " stu.student_code 	student_code,"+
						 " stu.identity_number 	identity_number,"+
						 " stu.course_name 		course_name,"+
						 " stu.course_code 		course_code,"+
						 " stu.finished			finished"+
					" from ec_oe_exam_student stu "+
					" where stu.exam_id='"+examId);
		StringBuffer studentCodesStringBuffer = new StringBuffer();
		for(String studentCode:studentCodes){
			studentCodesStringBuffer.append("'"+studentCode+"',");
		}
		String studentCodesString = studentCodesStringBuffer.toString();
		studentCodesString = studentCodesString.substring(0, studentCodesString.lastIndexOf(","));
		sql.append(" and stu.student_code IN ("+studentCodesString+")");
		
		sql.append(" and stu.course_code LIKE '%"+courseCode+"'");
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudent>() {
			@Override
			public ExamStudent mapRow(ResultSet rs, int arg1) throws SQLException {
				return getExamStudentByResultSet(rs);
			}
		});
	}

	@Override
	public List<ExamStudent> findExamStudents(Long examId,List<String> identityNumbers) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "+
						 " stu.id 				id,"+
						 " stu.exam_id			exam_id,"+
						 " stu.student_name 	student_name,"+
						 " stu.student_code 	student_code,"+
						 " stu.identity_number 	identity_number,"+
						 " stu.course_name 		course_name,"+
						 " stu.course_code 		course_code,"+
						 " stu.finished			finished"+
					" from ec_oe_exam_student stu "+
					" where stu.exam_id = "+examId);
		StringBuffer numberString = new StringBuffer();
		for(String number:identityNumbers){
			numberString.append("'"+number+"',");
		}
		String numbers = numberString.toString();
		numbers = numbers.substring(0, numbers.lastIndexOf(","));
		sql.append(" and stu.identity_number in ("+numbers+")");
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudent>() {
			@Override
			public ExamStudent mapRow(ResultSet rs, int arg1) throws SQLException {
				return getExamStudentByResultSet(rs);
			}
		});
	}


	@Override
	public List<OutletScore> queryExamScoreBy(Long rootOrgId, Long examId,String studentCode, String courseCode) {
		StringBuffer sql = new StringBuffer();
		sql.append("select "
	                +" s.id 						score_id,"
	                +" stu.student_name 			student_name,"
	                +" stu.student_code 			student_code,"
	                +" stu.identity_number 			identity_number," 
	                +" score.total_score 			total_score,"
	                +" score.objective_score 		objective_score,"
	                +" score.subjective_score 		subjective_score," 
	                +" data.start_time				start_time,"
	                +" data.end_time 				end_time," 
	                +" data.face_success_percent 	succ_percent,"
	                +" data.is_illegality 			is_illegality "
	                +" from  ec_oe_exam_student stu" 
	                +" left join ec_oe_exam_record record on stu.exam_student_id = record.exam_student_id" 
	                +" left join ec_oe_exam_record_data data on record.id = data.exam_record_id" 
	                +" left join ec_oe_exam_score score on data.id = score.exam_record_data_id" 
	                +" where stu.exam_id = "+examId
	                +"	and	 stu.student_code='"+studentCode+"'"
	                +"	and	 stu.course_code LIKE '%"+courseCode+"'");
		log.info(sql.toString());
		return  jdbcTemplate.query(sql.toString(), new RowMapper<OutletScore>() {
			@Override
			public OutletScore mapRow(ResultSet rs, int arg1) throws SQLException {
				return getExamScoreByIdResult(rs);
			}
		});
	}
	
}

